package erkka;

import javax.swing.undo.*;
import javax.swing.event.*;
import java.util.*;

/**
 * <p>Title: Erkka-Projekti</p>
 * <p>Description: This class is part of undo. </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Erkka</p>
 * @author Antti Enqvist & Juho Yli-Honkola
 * @version 1.0
 */

class AddEdit extends AbstractUndoableEdit {
  private DrawingArea objectList = null;
  private BaseObject object = null;

  /** */
  public AddEdit( DrawingArea objs, BaseObject obj ) {
    objectList = objs;
    object = obj;
  }

  /** True if it is still possible to redo this operation */
  public boolean canRedo() { return true; }

  /** True if it is still possible to undo this operation */
  public boolean canUndo() { return true; }

  /**
   * Provide a localized, human readable description of this edit suitable for
   * use in, say, a change log.
   */
  public String getPresentationName() {
    return "Add";
  }

  /** Re-apply the edit, assuming that it has been undone. */
  public void redo() throws CannotRedoException {
    objectList.AddObj( object, object.getX(), object.getY() );
    objectList.repaint();
  }

  /** Undo the edit that was made. */
  public void undo() throws CannotUndoException {
    objectList.removeObj( object );
    objectList.repaint();
  }

}


class RemoveEdit extends AbstractUndoableEdit {
  private DrawingArea objectList = null;
  private Vector objects = null;

  /** */
  public RemoveEdit( DrawingArea area, Vector objs ) {
    objectList = area;
    objects = new Vector( objs );
  }

  /** True if it is still possible to redo this operation */
  public boolean canRedo() { return true; }

  /** True if it is still possible to undo this operation */
  public boolean canUndo() { return true; }

  /**
   * Provide a localized, human readable description of this edit suitable for
   * use in, say, a change log.
   */
  public String getPresentationName() {
    return "Remove";
  }

  /** Re-apply the edit, assuming that it has been undone. */
  public void redo() throws CannotRedoException {
    objectList.setFocusVector( objects );
    objectList.removeSelectedObjects();
    objectList.repaint();
  }

  /** Undo the edit that was made. */
  public void undo() throws CannotUndoException {
    for (int i=0; i < objects.size(); i++) {
      BaseObject object = (BaseObject)objects.get(i);
      objectList.AddObj( object, object.getX(), object.getY() );
      objectList.setFocusVector( objects );
      objectList.repaint();
    }
  }
}


class MoveEdit extends AbstractUndoableEdit {
  private DrawingArea objectList = null;
  private Vector objects = null;
  private int mx = 0;
  private int my = 0;
  private int old_mx = 0;
  private int old_my = 0;

  /** */
  public MoveEdit( DrawingArea area, Vector objs, int x, int y) {
    objectList = area;
    objects = objs;
    mx = objectList.getFocusTableLeft();
    my = objectList.getFocusTableTop();
    old_mx = x;
    old_my = y;
  }

  /** True if it is still possible to redo this operation */
  public boolean canRedo() { return true; }

  /** True if it is still possible to undo this operation */
  public boolean canUndo() { return true; }

  /**
   * Provide a localized, human readable description of this edit suitable for
   * use in, say, a change log.
   */
  public String getPresentationName() {
    return "Move";
  }

  /** Re-apply the edit, assuming that it has been undone. */
  public void redo() throws CannotRedoException {
    objectList.setFocusVector( objects );
    objectList.setSelectedObjectsPos( mx, my );
//    System.out.println("Redo move");
  }

  /** Undo the edit that was made. */
  public void undo() throws CannotUndoException {
    objectList.setFocusVector( objects );
    objectList.setSelectedObjectsPos( old_mx, old_my );
//    System.out.println("Undo move");
  }

}
