package erkka;

import javax.swing.*;
import java.awt.Color;
import java.util.Vector;
import java.awt.event.*;

/**
*<p>Title:        <b>Erkka-Projekti</b></p>
*<p>Description:  Information panel shows selected symbol data.</p>
*<p>Copyright:    Copyright (c) 2002 Erkka-projekti</p>
*<p>Modified:     Jukka Mntyl 2005. Update data and repaint when combobox or checkbox changes</p>
*@version 1.2
*/

public class InformationPanel extends JPanel {

  BaseObject currSymbol;
  Vector pNames = new Vector();
  Vector pDatas = new Vector();
  SymbolFactory sFactory;
  public InformationPanel() {
    setLayout(null);
    for(int i=0;i<15;i++){
      pNames.add(new JLabel("testi"));
      pDatas.add(new PropDataField());
    }
    JLabel pComp; PropDataField pData;
    for (int i=0;i<pNames.size();i++){
        pComp = (JLabel)pNames.get(i);
        add( pComp );  pComp.setBounds(3, 60+(24*i), 65, 22);
        pData = (PropDataField)pDatas.get(i);
        add( pData );  pData.setBounds(68, 60+(24*i), 85, 22);
        pData.setIndex(i);
        pComp.setVisible(false);
        pData.setVisible(false);
        pData.addKeyListener(new java.awt.event.KeyAdapter() {
          public void keyReleased(KeyEvent e) { pData_keyType(e);}
       });
    }
  }

/**
 * Set SymbolFactory.
 * @param symbF
 */
  public void setSymbolF(SymbolFactory symbF){
    sFactory=symbF;
  }
/**
 * Remove all combo and checkboxes.
 */
  public void removeComboAndCheckBoxes(){
   for (int i=0;i<getComponentCount();i++){
    Object o=getComponent(i);
    if(o instanceof JComboBox || o instanceof JCheckBox){
      remove(i); i--;}
   }

  }
/**
 * Sets symbol data data and field name. If symb is null
 * then hide all fields.
 * @param symb
 */
  public void setSymbolData(BaseObject symb){
    if(currSymbol !=null ) saveOldSymbolData();
    removeComboAndCheckBoxes();
    currSymbol = symb;
    if(symb == null){
      hideProperties(0);
      return;
    }
    hideProperties(symb.getFieldsCount());
    // normal text field or combo or check box
    for(int i=1;i<symb.getFieldsCount();i++){
        ((JLabel)pNames.get(i)).setText(symb.getFieldName(i));
        if(symb.getFieldDataType(i)!=null &&(symb.getFieldDataType(i)).length()>1)
            createSelection(i,symb.getFieldDataType(i));
        else
          {((PropDataField)pDatas.get(i)).setText(symb.getFieldData(i));
          ((PropDataField)pDatas.get(i)).setIndex(i);
        }
    }
  }
  /**
   * When current symbol change, first save old symbol data.
   */
  public void saveOldSymbolData(){
    for(int i=1;i<currSymbol.getFieldsCount();i++){
      if(currSymbol.getFieldDataType(i)!=null && currSymbol.getFieldDataType(i).length()>1){
        currSymbol.SetProperty(getSelectionData(i),i);
        String name=((JLabel)pNames.get(i+1)).getText();
        if(!name.equalsIgnoreCase(currSymbol.getFieldName(i+1)))
          currSymbol.setNewField(name,null,null,i+1);


      }
      else
        currSymbol.SetProperty (((PropDataField)pDatas.get(i)).getText(),i);

    }
  }
  /**
   * Return data on combo or checkboxes.
   * @param i Field index.
   * @return
   */
  public String getSelectionData(int i){
    for(int j=0;j<getComponentCount();j++){
      if( getComponent(j) instanceof JCheckBox){
        JCheckBox b = (JCheckBox)getComponent(j);
        if(b.getName().equalsIgnoreCase(String.valueOf(i)))
          if(b.isSelected())
            return "true";
          else
            return " ";
      }
      if (getComponent(j) instanceof JComboBox){
        JComboBox cb = (JComboBox) getComponent(j);
        if(cb.getName().equalsIgnoreCase(String.valueOf(i)))
          return (String)cb.getSelectedItem();
      }
    }
    return "error";
  }
  /**
   * Create combo or checkbox right place.
   * @param i Place
   * @param type  Tells what kind of box create. (Type must be defined in
   * config file)
   */
  public void createSelection(int i,String type){
    PropDataField p=(PropDataField)pDatas.get(i);
    p.setVisible(false);

    JComponent selectionBox;
    // SymbolFactory create combo or checkbox.
    selectionBox=sFactory.createSelection(type);
    add(selectionBox);
    p.setComponentOverSelf(selectionBox);
    if(selectionBox instanceof JComboBox)
      installData((JComboBox)selectionBox,i);
    if(selectionBox instanceof JCheckBox)
      installData((JCheckBox)selectionBox,i);


  }
  /**
   * Set right item selected in combo.
   * @param box
   * @param i Symbol field where data read.
   */
  public void installData(JComboBox box,int i){

    box.setSelectedItem(currSymbol.getFieldData(i));
    box.setName(String.valueOf(i));
    box.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e){
        comboBoxChoiceSelected(e);}
    });
    box.revalidate();
  }
  /**
   * Select checkbox selected if field data is "true"
   * @param box
   * @param i field number
   */
  public void installData(JCheckBox box,int i){
    if(currSymbol.getFieldData(i).equalsIgnoreCase("true"))
      box.setSelected(true);
    else
      box.setSelected(false);
    box.setName(String.valueOf(i));
    box.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent e){
        String text;
        JCheckBox box=(JCheckBox)e.getSource();
        if(box.isSelected())
          text = "true";
        else
          text = " ";

        int index = Integer.parseInt(box.getName());

        // update data
        currSymbol.SetProperty(text, index);
        //currSymbol.repaint();
      }
    });
  }

  public void pData_keyType(KeyEvent e){
    PropDataField temp;
    temp=((PropDataField)e.getComponent());
    if(currSymbol==null)
      return;
    currSymbol.SetProperty(temp.getText(), temp.getIndexInT());
    currSymbol.repaint();
  }

  /**
   * Hide textfields that not need.
   * @param Count
   */
  public void hideProperties(int Count){

    for(int i=1;i<=(pNames.size()-Count);i++){
      ((JComponent)pNames.get(pNames.size()-i)).setVisible(false);
      ((JComponent)pDatas.get(pNames.size()-i)).setVisible(false);
    }
    for(int i=0;i<Count;i++){
      ((JComponent)pNames.get(i)).setVisible(true);
      ((JComponent)pDatas.get(i)).setVisible(true);
    }

      ((JComponent)pNames.get(0)).setVisible(false);
      ((JComponent)pDatas.get(0)).setVisible(false);
  }

  /**
   * When some selected in combobox, find out if current selection need
   * extra data field(eg. VARCHAR NEED length field) and generates it.
   * @param e
   */
  public void comboBoxChoiceSelected(ActionEvent e){
  JComboBox box=(JComboBox)e.getSource();
  String text =(String)box.getSelectedItem();
  Proper proper= sFactory.getChoiceProper(text);
  int index =Integer.parseInt(box.getName());

  // update data and paint
  currSymbol.SetProperty(text, index);
  currSymbol.repaint();

  // new fields
  removeOldExtra(box,index);
  if(proper ==null){ return;}

  // move all fields one step down
  moveDown(24,index);
// move ends
  PropDataField field =new PropDataField();
  field.setText(proper.getPData());
  pDatas.insertElementAt(field,index+1);
  add(field);
  field.setBounds(68, 60+(24*(index+1)), 85, 22);
  JLabel lab=new JLabel(proper.getPName());
  pNames.insertElementAt(lab,index+1);
  add(lab);
  lab.setBounds(3, 60+(24*(index+1)), 65, 22);

  currSymbol.setNewField(proper.getPName(),proper.getPData(),proper.getPDType(),index+1);

  }
  /**
   * Find out extra datafields(need to know that we can remove it)
   * @param box
   * @param index
   * @return
   */
  public boolean isThereOldExtraField(JComboBox box,int index){
    Proper p ;

    for(int i=0;i<box.getItemCount();i++ ){
      p=sFactory.getChoiceProper((String)box.getItemAt(i));
      if(p!=null)
        if(p.getPName().equalsIgnoreCase(((JLabel)pNames.get(index+1)).getText()))
          return true;
    }
    return false;
  }
/**
 * Remove old exrafield and insert new value to symbol and move fields up.
 * @param box
 * @param index place where field remove
 */
  public void removeOldExtra(JComboBox box,int index){
    if(!isThereOldExtraField(box,index))
      return;

    currSymbol.removeField(index+1);
    currSymbol.SetProperty((String)box.getSelectedItem(),index);
    remove((JComponent)pDatas.get(index+1));
    pDatas.remove(index+1);
    remove((JComponent)pNames.get(index+1));
    pNames.remove(index+1);
    moveDown(-24,index);

  }
  /**
   * Move fields up or down start by given index. Update index ++ or --
   * depends on direction of move(move>0 ++  move<0 --).
   * @param move
   * @param index
   */
  public void moveDown(int move,int index){
    int index2=move;

    if(move<0)index2=-1;
    else if(move>0)index2=1;

    for(int i =index+1;i<pDatas.size();i++){
      PropDataField pF=(PropDataField)pDatas.get(i);
      pF.setIndex(pF.getIndexInT()+index2);
      pF.moveUpOrDown(move);

    }
    for(int i =index+1;i<pNames.size();i++){
      JComponent c=(JComponent)pNames.get(i);
      c.setBounds(c.getX(),c.getY()+move,c.getWidth(),c.getHeight());
    }
  }

   public void setFocus(){
    ((JTextField)pDatas.get(1)).requestFocus();
   }
}
