package erkka;

import java.util.Collection;
import java.util.ArrayList;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import java.io.*;
import java.util.*;
import java.util.zip.*;

/**
 * <p>Title: Connects to mdb (Access)/odb (OO 2.0 HSQLDB) database or using ODBC/JDBC and executes SQL query batch</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * <p>Company: jyu</p>
 * @author Jukka Mntyl
 * @version 1.21 Unzips .odb-files, connects to db, and zips back again when disconnected. A bug fixed.
 */

public class ODBCQuery {

  private String ajuri, URL, nick, passwd;
  private Connection yhteys = null;
  private final int DEFAULTDISPLAYSIZE = 20;
  private int displaysize = DEFAULTDISPLAYSIZE;
  private String dbpath, suffix, tmppath;

  private Collection asiakkaat = new ArrayList();

  public ODBCQuery(String localdatabase, String suffix) {
    dbpath = localdatabase;
    this.suffix = suffix;

    if (suffix.equals("mdb")) {
      ajuri = "sun.jdbc.odbc.JdbcOdbcDriver";
      URL = "jdbc:odbc:Driver={Microsoft Access Driver (*.mdb)};ExtendedAnsiSQL=1;DBQ=" + localdatabase;
      nick = "";
      passwd = " ";
    }
    if (suffix.equals("odb")) {
      ajuri = "org.hsqldb.jdbcDriver";
      URL = "jdbc:hsqldb:";
      nick = "sa";
      passwd = "";
    }
  }

  /**
   * <p>java -Djdbc.drivers=yourJdbcDriver -classpath driverPath -jar erkka.jar</p>
   */
  public ODBCQuery(String ajuri, String URL, String nick, String passwd) {
    this.ajuri = ajuri;
    this.URL = URL;
    this.nick = nick;
    this.passwd = passwd;
    suffix = "";
  }

  public void connect() throws Exception{
    try {
      if (suffix.equals("odb")) {
        unzipODB();
        URL = URL + tmppath + File.separator + "db";
      }
      // JDBC-ODBC-ajurin lataaminen, tietokantayhteyden muodostaminen ja
      // statementin luonti sql-kyselyj varten
      Class.forName(ajuri);
      yhteys = DriverManager.getConnection(URL, nick, passwd);
    }
    catch (ClassNotFoundException e) {
      throw new Exception("Error. Cannot find database driver: " + e.getMessage());
    }
    catch (SQLException e) {
      throw new Exception("Error. Problem with connection: " + e.getMessage());
    }
    catch (Exception e) {
      throw new Exception("Error. Cannot unzip .odb-file: " + e.getMessage());
    }


  }

  public boolean isConnected() {
    return (yhteys != null);
  }

  public void disconnect() throws Exception {
    if (yhteys != null) {
      if (suffix.equals("odb")) {
        try {
          executeQuery("SHUTDOWN",yhteys.createStatement());
          zipODB();
        } catch (Exception e) {
          throw new Exception("Error. Can't finalize database: " + e.getMessage());
        }
      }
      yhteys.close();
      yhteys = null;
    }
  }

  public String executeQueryBatch(String sqlquery) throws Exception {
    return executeQueryBatch(sqlquery,';');
  }

  private int countChars(StringBuffer jono, char c) {
    int count = 0;
    for (int i=0;i<jono.length();i++)
      if (jono.charAt(i) == c)
        count++;
    return count;
  }

  private String getShortQueryCommand(String query) {
    int komennonpituus[] = {query.indexOf('\n'), query.indexOf('('),
                            query.indexOf("WHERE"), query.indexOf("where"),
                            query.indexOf("VALUES"), query.indexOf("values"),
                            query.indexOf("SET"), query.indexOf("set")};
    int alkupituus = query.length();
    for (int i=0;i<komennonpituus.length;i++) {
      if (komennonpituus[i] != -1 && alkupituus > komennonpituus[i])
        alkupituus = komennonpituus[i];
    }
    return query.substring(0,alkupituus).trim();
    /*
    int komennonpituus = ;
    if (komennonpituus == -1 || komennonpituus > )
      komennonpituus = query.indexOf('(');
    if (komennonpituus == -1 || komennonpituus > )
 */
  }

  // voisi mys ajaa oikeana batchin
  //   stmt.addBatch(queries[i]);
  //    stmt.executeBatch();
  public String executeQueryBatch(String sqlquery, char separator) throws Exception {
    // poistetaan sql-kommentit
    StringBuffer querybuffer = new StringBuffer(sqlquery);
    boolean merkkijonossa = false;
    boolean kommentissa = false;
    char c;
    for (int i=0;i<querybuffer.length();i++) {
      c = querybuffer.charAt(i);
      if (c == '\'')
        merkkijonossa = merkkijonossa ? false : true;
      if (!merkkijonossa) {
        if (!kommentissa) {
          if (c == '-')
            if (i + 1 < querybuffer.length())
              if (querybuffer.charAt(i + 1) == '-') {
                // poistaa kaksi viivaa
                querybuffer.deleteCharAt(i);
                querybuffer.deleteCharAt(i);
                kommentissa = true;
              }
        }
        else {
          if (c == '\n')
            kommentissa = false;
          else {
            querybuffer.deleteCharAt(i);
            i--; //bug fixed 22.2.2006
          }
        }
      }
    }


    // hajotetaan eri kyselyiksi
    // hajottaa mys sellaiset joissa ; keskell ''-merkkijonoa
    String[] queries = querybuffer.toString().split(";"); // bug fixed 22.2.2006

    StringBuffer query = new StringBuffer("");
    Statement stmt = null;
    StringBuffer resultbuffer = new StringBuffer("");
    displaysize = DEFAULTDISPLAYSIZE;
    try{
      stmt = yhteys.createStatement();

      for (int i=0;i<queries.length;i++) {
        query.setLength(0);
        query.append(queries[i]);
        // tarkistetaan, ett kysely ei ole splittautunut vrlt kohdalta
        // yhdistetn, jos vaillinainen mr '-merkkej
        while (countChars(query,'\'')%2 == 1) {
          i++;
          if (i < queries.length)
            query.append(queries[i]);
          else
            break;
        }
        String executablequery = query.toString().trim();
        if (executablequery.length() > 3) {
          resultbuffer.append(executeQuery(executablequery, stmt));
          resultbuffer.append("\n\n");
        }
      }
      return resultbuffer.toString();
    } catch (SQLException e) {
        throw new Exception(e.getMessage());
    } finally {
      if (stmt != null) try { stmt.close(); } catch (Exception e) {;}
    }
  }

  public int getDisplaySize() {
    return displaysize;
  }

  private StringBuffer executeQuery(String query, Statement stmt) throws Exception {


    ResultSet rs = null;
    int updateCount = 0;
    StringBuffer qresultbuffer = new StringBuffer("");
    StringBuffer viivabuffer = new StringBuffer(""); // kentn nimien ja datan erotin
    int i = 0, len = 0;
    String s;
    try{
      if (stmt.execute(query)) {
        rs = stmt.getResultSet();
        ResultSetMetaData rsmd = rs.getMetaData();
        if (rsmd.getColumnCount() > 0) {
          // otsikkotiedot
          s = rsmd.getColumnName(1);
          len = s.length();
          qresultbuffer.append(s);
          appendLine(viivabuffer, len);
        //  if (rsmd.getColumnName(1).length() > displaysize)
          //  displaysize = rsmd.getColumnName(1).length();
          for (i = 2; i <= rsmd.getColumnCount(); i++) {
            s = rsmd.getColumnName(i);
            len = s.length();
            //if (rsmd.getColumnName(i).length() > displaysize)
            //  displaysize = rsmd.getColumnName(i).length();
            qresultbuffer.append('\t' + rsmd.getColumnName(i));
            viivabuffer.append('\t');
            appendLine(viivabuffer, len);
            if (len > displaysize)
              displaysize = len;
          }
          qresultbuffer.append('\n');
          qresultbuffer.append(viivabuffer);
          qresultbuffer.append('\n');
          // tiedot
          while (rs.next()) {
            s = rs.getString(1);
            len = s.length();
            qresultbuffer.append(s);
            if (len > displaysize)
              displaysize = len;
            //if (rs.getString(1).length() > displaysize)
             // displaysize = rs.getString(1).length();
            for (i = 2; i <= rsmd.getColumnCount(); i++) {
              //if (rs.getString(i).length() > displaysize)
               // displaysize = rs.getString(i).length();
               s = rs.getString(i);
               len = s.length();
               qresultbuffer.append('\t' + s);
               if (len > displaysize)
                 displaysize = len;
            }
            qresultbuffer.append('\n');
          }
        }
      } else {
        updateCount = stmt.getUpdateCount();
        if (updateCount >= 0) {
          qresultbuffer.append(getShortQueryCommand(query) + " completed successfully. " + updateCount + " rows affected.");
        } else {
          qresultbuffer.append(getShortQueryCommand(query) + " completed successfully. No results were returned.");
        }
      }
      return qresultbuffer;

    } catch (SQLException e) {
        //int pos = query.indexOf(" ");
        // query.substring(0, pos > query.length() ? query.length() : pos)
        return new StringBuffer("Error in query " + getShortQueryCommand(query) + " :\n" + e.getMessage());
    } finally {
      if (rs != null) try { rs.close(); } catch (Exception e) {;}
    }
  }

  private void appendLine(StringBuffer b, int count) {
    for (int i=0;i<count;i++)
      b.append('-');
  }


  private void copyInputStream(InputStream in, OutputStream out)
  throws IOException
  {
    byte[] buffer = new byte[1024];
    int len;

    while((len = in.read(buffer)) >= 0)
      out.write(buffer, 0, len);

    in.close();

  }

  /**
   * Purkaa ODB-muotoisen zip-paketin temppiin, muuttaa tietokantatiedostojen nimet db-alkuisiksi
   * @throws IOException
   */
  private void unzipODB() throws IOException {
    Enumeration entries;
    ZipFile zipFile;
    String entryname;
    File tmpfile;
    tmppath = java.lang.System.getProperty("java.io.tmpdir") + "database" + java.lang.System.currentTimeMillis();
    File tmpdir = new File(tmppath);
    tmpdir.mkdir();

      zipFile = new ZipFile(dbpath);

      entries = zipFile.entries();

      while(entries.hasMoreElements()) {
        ZipEntry entry = (ZipEntry)entries.nextElement();
        entryname = entry.getName();

        if (entryname.equals("database/data") || entryname.equals("database/properties")
            || entryname.equals("database/script") || entryname.equals("database/backup")) {

        tmpfile = new File(tmpdir.getAbsolutePath() + File.separator + "db." + entryname.substring(entryname.indexOf('/') + 1));
        BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(tmpfile));
        copyInputStream(zipFile.getInputStream(entry), out);
        out.close();
        }
      }
      zipFile.close();

  }

  /**
   * Kopioi vanhan ODB-tiedoston, mutta tietokantatiedostot temp-hakemiston uusilla tiedostoilla
   */
  private void zipODB() throws IOException {

      ZipFile zipFile = new ZipFile(dbpath);
      FileInputStream f;
      File file;
      String entryname;

      ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(dbpath + ".tmp"));

      Enumeration entries = zipFile.entries();
      while(entries.hasMoreElements()) {
         ZipEntry entry = (ZipEntry)entries.nextElement();
         entryname = entry.getName();

         if (entry.isDirectory()) {
           continue;
         }

         if (entryname.equals("database/data") || entryname.equals("database/properties")
             || entryname.equals("database/script") || entryname.equals("database/backup")) {
           continue;
         } else {

           zos.putNextEntry(entry);
           copyInputStream(zipFile.getInputStream(entry), zos);
         }
      }
      String[] dbentries = {"data", "properties", "script", "backup" };
      for (int i=0;i<dbentries.length;i++) {
        file = new File(tmppath + File.separator + "db." + dbentries[i]);
        if (file.exists()) {
          f = new FileInputStream(file);
          zos.putNextEntry(new ZipEntry("database/" + dbentries[i]));
          copyInputStream(f, zos);
          zos.closeEntry();
        }
      }

      zos.close();
      zipFile.close();

      file = new File(dbpath);
      File oldfile = new File(dbpath + ".old");
      if (oldfile.exists())
        oldfile.delete();

      if (file.renameTo(oldfile)) {
        file = new File(dbpath + ".tmp");
        if (!(file.renameTo(new File(dbpath))))
          throw new IOException("Can't rename " + dbpath + ".tmp");
      } else {
        throw new IOException("Can't rename " + dbpath );
      }
  }

}
