package erkka;

import java.io.*;
import java.lang.String;
import java.util.StringTokenizer;
import java.util.Vector;
/**
 * <p>Title:</p>
 * <p>Description: Relation knows all information about a relation(table):
 *              tablaName, primarykey, attributes and the list of the lines,
 *              whose type is Reference.</p>
 * <p>Copyright:    Copyright (c) 2002</p>
 * <p>Company: University of Jyvskyl</p>
 * @author HLP
 * @version 1.2 Jukka Mntyl findAttributes - parsing NUMERIC(m,n)-type attributes fixed
 */

public class Relation {
  /**Name of the table */
  private String tableName = null;
  /**Tables primarykey or -keys */
  private Vector primarykey = new Vector();
  /**Attributes of teh table as Vector */
  private Vector attributes = new Vector();
  /**Lines, whice table owns */
  private Vector lines = new Vector(); //references

  /**constructor creates new Relation*/
  public Relation() {
  }

  /**@return tableName */
  public String getTableName () { return tableName; }
  /**@return primarykey */
  public Vector getPrimarykey() { return primarykey; }
  /**@return attributes */
  public Vector getAttributes() { return attributes; }
  /**@return lines */
  public Vector getLines() { return lines; }

/**
 * <p>Is primarykey or not</p>
 * @param attrname Name of attribute
 * @return true if is primarykey and false if not
 */
  public boolean isPrimaryKey( String attrname ) {
    for (int i=0; i<primarykey.size(); i++) {
      if (primarykey.get(i).equals(attrname))
        return true;
    }
    return false;
  }

/**
 * <p>Remove all enter from <code>CREATE TABLE</code> to ); -character.</p>
 * @param breader
 * @param line line whice is readed, not include <code>CREATE TABLE</code> -word
 * @return tempLine: is line whice includes one table
 */
  public String removeEnterInCreateTable(BufferedReader breader, String line) {
    String tempLine = line;
    try {
      while (tempLine.indexOf(");")==-1)
        tempLine = tempLine+breader.readLine();
    } catch (IOException ioe) {
      ioe.printStackTrace();
    }
    return tempLine;
  }
/**
 * <p>Remove extra spaces. After this method there can be only
 * one space between two words.
 * Remove all spaces at the begin of the line.</p>
 * @param line whice we are reading
 * @return stemp line, whice extra space are removed
 */
  public String removeExtraSpace(String line) {
    char temp = ' ';
    String stemp = new String("");
    for (int i=0;i<line.length();i++) {
      if ((line.charAt(i) == ' ')&&( temp == ' ') ) {;}
      else if (line.charAt(i) == '\t'){
        if (temp != ' ' ) {stemp = stemp+(" ");}
      }
      else {
        stemp = stemp+String.valueOf(line.charAt(i));
      }
      if (line.charAt(i) == '\t') temp = ' ';
      temp = line.charAt(i);
    }
    return stemp;
  }
/**
 * <p>creates <code>CREATE TABLE</code> by placing all information at the same line.
 * The line is devided to attributeLine and endLine, then is asked
 * functions <code>findAttributes</code> and <code>findConstraints</code>
 * 1.line:<code>CREATE TABLE</code>
 * 2.line:<code>Osasto(OsastoID INTEGER NOT NULL,OsastoNimi VARCHAR(32)....</code></p>
 * @param breader
 * @param line the first index of line is the first character
 *              after <code>CREATE TABLE</code> and the last character is );
 */
  public void createTable(BufferedReader breader,String line) throws Exception {
    String tempLine = removeEnterInCreateTable(breader, line);
    tempLine = removeExtraSpace(tempLine);
    tableName = tempLine;
    String word = "CONSTRAINT";
    int indexBracket = tempLine.indexOf("(");
    int indexEmpty = tempLine.indexOf(" ");
    int indexEnd;
    int indexWord;
    String endLine = null;
    String attriLine = null;
      // find tableName
    if ( indexBracket == 0 ) { throw new Exception("A relation has no name");}
    if ( indexEmpty == -1 ) { //there is no empty character at line
      tableName = tableName.substring(0,indexBracket);
    }
    else if ( indexEmpty < indexBracket ) {//there is one empty character before (-character
      tableName = tableName.substring(0,indexEmpty);
    }
    else {
      tableName = tableName.substring(0,indexBracket);
    }

    tempLine = tempLine.substring(indexBracket+1); //(-character is removed
    indexEmpty = tempLine.indexOf(" "); //next space
    if ( indexEmpty == 0 ) // if space before next word, in the index of 0. not -1
      tempLine = tempLine.substring(indexEmpty+1);
    indexWord = tempLine.indexOf(word);

      // if CONSTRAINT is the first word in the create table
    if ( indexWord == 0 ) {
      //System.out.println("There is no attributes in:"+tableName);
      endLine = tempLine.substring(indexWord); // from The C-character of the first CONSTRAINT-word
      findConstraints(endLine);
      return;
    }
      //not include CONSTRAINT and attributes
    else if (tempLine.length() < 4) { // CREATE TABLE ( is removed, there can
      throw new Exception("Create table is empty");//be empty character and );
      //return;                                            //also 3 character
    }
      //not include CONSTRAINT
    else if (indexWord < 0) {
      attriLine = tempLine;//to Constraint-word, not include comma before the word
      int indexPoint = tempLine.indexOf(",");
      indexEnd = tempLine.indexOf(");");
      if (indexPoint < indexEnd) // if there is , and ); at end of the line
        if (indexPoint >= indexEnd-2)
        attriLine = attriLine.substring(0,indexPoint);
      findAttributes(attriLine);
      throw new Exception("There is no "+word);
      //return;
    }
      // if there is both attributes and CONSTRAINTS
    else { //attriLine don't get have comma in the end of the line
      if (tempLine.charAt(indexWord-1) == ' ')
        attriLine = tempLine.substring(0, indexWord-2); //to Constraint-word, not include comma before the word
      else
        attriLine = tempLine.substring(0, indexWord-1); //to Constraint-word, not include comma before the word
      findAttributes(attriLine);
      endLine = tempLine.substring(indexWord); //from The C-character of the first CONSTRAINT-word
      findConstraints(endLine);
    }
  }
  /**
   * Finds first of specified char in line string but ignore chars between start->end
   * @return int index of char
   */
  private int findIgnoreBetween(char seek, char start, char end, String line) {
    int seekpos = -1;
    int startpos = line.indexOf(start);
    int endpos = line.indexOf(end);
    // jos startpos > endpos niin ei ht
    do {
      seekpos = line.indexOf(seek, seekpos+1);
    } while ( (seekpos != -1) && (startpos <= seekpos) && (seekpos <= endpos));
    return seekpos;
  }
/**
 * <p>find attributes</p>
 * <p>modified by Jukka Mntyl 2005. Bugfix for attributes like NUMERIC(7,2)</p>
 * @param attriLine AttriLine begin with first attribute and end with word
 *                  before <code>CONSTRAINT</code>. The first character of the attriLine
 *                  can't be empty and the last don't get be comma.
 */
  public void findAttributes(String attriLine) {
    int commapos = findIgnoreBetween(',','(',')',attriLine);
    int spacepos = attriLine.indexOf(" ");
    String temp = attriLine;
    attributes.add(findFirstWord(temp));
    while (commapos != -1) {
      temp = temp.substring(commapos+1);  //is removed to comma
      spacepos = temp.indexOf(" ");       //next space
      if ( spacepos == 0 )                //if there is space before the next word
        temp = temp.substring(spacepos+1);//space deleted
      attributes.add(findFirstWord(temp));
      //commapos = temp.indexOf(",");       //next ','
      commapos = findIgnoreBetween(',','(',')',temp);
    }
  }
/**
 * <p>find constrains</p>
 * @param endLine the first character of the endLine is C (<code>CONSTRAINT</code>)
 *                 and the last character is );
 */
  public void findConstraints(String endLine) throws Exception {
    String wordConstraint = "CONSTRAINT";
    String wordPrimary = "PRIMARY KEY";
    String wordForeign = "FOREIGN KEY";
    int indexConstraint = endLine.indexOf(wordConstraint);
    int indexPrimary;
    int indexForeign;
    int firstBracket;
    int endBracket;
    String temp = endLine;

    while (indexConstraint != -1) { //untils there is no CONSTRAINTS
      temp = temp.substring(indexConstraint+wordConstraint.length());//constraint is removed
      indexPrimary = temp.indexOf(wordPrimary);

      //if exist PRIMARY KEY
      if (indexPrimary != -1) {
        firstBracket = temp.indexOf("(");
        endBracket = temp.indexOf(")");
        String line = temp.substring(firstBracket+1,endBracket);//remove brackets
        Vector primarykey = IncludeOfBracketAsVector(line);
        temp = temp.substring(indexPrimary+wordPrimary.length());
      }
      indexForeign = temp.indexOf(wordForeign);

      //if exist FOREIGN KEY
      if (indexForeign != -1) {
        temp = temp.substring(indexForeign+wordForeign.length());
        int indexBracket = temp.indexOf("CONSTRAINT"); //next constraint, jm: comma is not safe
        int indexEnd = temp.indexOf(");");//or end
        String refTemp = temp;
        if ((indexBracket < indexEnd) && (indexBracket > 0))
          refTemp = refTemp.substring(0,indexBracket-1);
        else refTemp = refTemp.substring(0,indexEnd);

        // jm: parsitaan jo tss sill voi tulla useita viivoja
        // jos kyseess on viive yhdistettyyn avaimeen
        String tmp = refTemp;
        String wordReference = "REFERENCES";
        int indexReference = refTemp.indexOf(wordReference);
        firstBracket = refTemp.indexOf("(");
        endBracket = refTemp.indexOf(")");
        String sourceattributes[] = tmp.substring(firstBracket+1,endBracket).split(",");//remove brackets

        indexReference = tmp.indexOf(wordReference);
        tmp = tmp.substring(indexReference+wordReference.length()+1);
        int index = tmp.indexOf(" ");
        String tableTarget2 = tmp.substring(0,index);

        firstBracket = tmp.indexOf("(");
        endBracket = tmp.indexOf(")");
        String targetattributes[] = tmp.substring(firstBracket+1,endBracket).split(",");//remove brackets
        //String cascade = temp.substring(endBracket+1);

        if (sourceattributes.length > 0 && sourceattributes.length == targetattributes.length) {
          for (int i=0;i<sourceattributes.length;i++) {
            Reference line = new Reference();
            line.setAttributes(sourceattributes[i], tableTarget2,
                               targetattributes[i]);
            lines.add(line);
          }
        } else {
          throw new Exception("Error in FOREIGN KEY -clause");
        }

        temp = temp.substring(refTemp.length());
      }
      indexConstraint = temp.indexOf("CONSTRAINT"); //next constraint
    }
  }
/**
 * <p>return the include of bracket as Vector.</p>
 * @param line
 * @return primarykey
 */
  public Vector IncludeOfBracketAsVector(String line) {
    if (line.indexOf(",")!=-1) {
      StringTokenizer mjonot = new StringTokenizer(line,",");
      while (mjonot.hasMoreTokens()) {
        primarykey.add(mjonot.nextToken());
      }
    }
    else primarykey.add(line);
    return primarykey;
  }
/**
 * <p>return the first word of the line.</p>
 * @param line firts character dont't get be empty
 * @return word to next space
 */
  public String findFirstWord(String line) {
    int index = line.indexOf(" ");
    return line.substring(0,index);
  }
}
