package erkka;

import javax.swing.JComponent;
import java.awt.Graphics2D;
import java.awt.font.*;
import java.awt.geom.*;
import java.awt.*;
import java.util.*;

/**
 * <p>Title: </p>
 * <p>Description: Rectangle that includes table name and attribute names</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @see Relation
 * @author Hanna-Leena Peltomaa
 * @version 1.0
 */

public class RelationBox extends Symbol {
  Relation relation;

  /** Horizontal space for each attribute */
  int attributeHeight = 15;
  /** The height of the header margin */
  int attributeStart = 25;
  /** How near should function {@link #GetNearest(Point foo, String attrname) GetNearest} return the HotSpot point */
  int lineSpace = 15;
  /** The width of longest text line (Name or attribute) */
  int maxWidth = 0;

  /** Precreated font for drawing the name  */
  Font nameFont = new Font("SansSerif",Font.PLAIN,16);
  /** Precreated font for drawing primary key attributes */
  Font pkFont = new Font("SansSerif",Font.BOLD,12);
  /** Precreated font for drawing attributes */
  Font attrFont = new Font("SansSerif",Font.PLAIN,12);
  /** Hashtable containg precreated textlayouts for attributes. Used to speed up drawing */
  Hashtable attributes = null;
  /** Height of attribute text */
  double textHeight = 0;

  public RelationBox(Relation relation) {
    this.relation = relation;
  }
/**
 * paint method
 * @param g
 */
  public void paint(Graphics g) {
    Graphics2D g2 = (Graphics2D)g;

    // Turn on antialiasing
    g2.setRenderingHint( RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);

    // Create font layout for attributes only on first run.
    if (attributes == null)
      createAttributeHash( g2 );

    FontRenderContext frc = g2.getFontRenderContext();
    TextLayout tl;

    g2.setColor(Color.black);

    // Draw the name of this table
    tl= new TextLayout(relation.getTableName(),  nameFont, frc);
    Rectangle2D r2d = tl.getBounds();
    tl.draw(g2, 5, (float)(r2d.getHeight()+3) );

    // Draw attributes
    int ypos = attributeStart;
    for (int i=0; i < relation.getAttributes().size(); i++)
    {
      tl = (TextLayout)attributes.get( (String)relation.getAttributes().get(i) );
      tl.draw(g2, 10,(float)(r2d.getHeight()+ypos) );
      ypos+=attributeHeight;
    }
    // Resize
    this.setBounds(getX(), getY(), maxWidth+20, (relation.getAttributes().size()*attributeHeight)+attributeStart +5);

    // Draw rectagle
    g2.drawRect(0,0,getWidth()-1,getHeight()-1);

    // Draw line between the name and attributes
    g2.drawLine(0, attributeStart, getWidth(), attributeStart );
  }

  /**
   *
   * @param attrname
   * @return Index of given attribute name in our internal representation.
   */
  int getAttributePos(String attrname) {
    for (int i=0; i < relation.getAttributes().size(); i++)
    {
      Vector v = relation.getAttributes();
      if ( v.get(i).equals(attrname) )
        return i;
    }
    return -1;
  }

  /**
   *
   * @param foo
   * @param attrname Name of attribute
   * @return Point near given attribute name which where relation line should be drawn.
   */
  public HotSpot GetNearest(Point foo, String attrname){
    if (attrname == null)
      return new HotSpot( getX(),getY() );

    int pos = getAttributePos( attrname );
    if (foo.getX() > getX())
        return new HotSpot(getX()+getWidth()+lineSpace,getY()+attributeStart+(pos*attributeHeight)+5 );

    return new HotSpot(getX()-lineSpace,getY()+attributeStart+(pos*attributeHeight)+5 );
  }

  /**
   *
   * @param cmpPoint X-coordinate for point which we are going to compare.
   * @return Integer value how much is the distance between our sides and the
   * <code>cmpPoint</code>.
   */
  int getVerticalSpace(int cmpPoint) {
    if (cmpPoint > getX())
        return -lineSpace;
    return lineSpace;
  }

  /**
   *
   * @return Name of this table defined in <code>Relation</code>-class belonging
   * to this class.
   */
  public String getName() {
    return relation.getTableName();
  }

  /**
   * This function creates <code>attributes</code>-hashtable which is used
   * in drawing of this component. The key of hashtable the name of the attribute
   * and the value is reference to a instance of TextLayout-class, which contains
   * the name text drawn with right font and style. Using this hashtable increases
   * the speed of drawing a lot.
   *
   * This function also defines the values of <code>maxWidth</code> and <code>
   * textHeight</code> -variables which defines the size of the box and the height
   * of attribute text.
   *
   * @param g2
   */
  public void createAttributeHash( Graphics2D g2 ) {
    FontRenderContext frc = g2.getFontRenderContext();
    TextLayout tl;

    attributes = new Hashtable();

    tl= new TextLayout(relation.getTableName(),  nameFont, frc);
    Rectangle2D r2d = tl.getBounds();

    maxWidth = (int)r2d.getWidth();

    // Iterate throught attribute names
    for (int i=0; i < relation.getAttributes().size(); i++)
    {
      String attrname = (String)relation.getAttributes().get(i);
      if (relation.isPrimaryKey(attrname))
      {
        tl= new TextLayout(attrname,  pkFont, frc);
        attributes.put( attrname, tl );
      }
      else
      {
        tl= new TextLayout(attrname,  attrFont, frc);
        attributes.put( attrname, tl );
      }

      r2d = tl.getBounds();
      if (r2d.getWidth() > maxWidth )
          maxWidth = (int)r2d.getWidth();
      textHeight = r2d.getHeight();
    }
  }
}